/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.util;

import java.util.*;

import edu.stanford.smi.protege.util.*;

/**
 *  Abstract superclass for lots of state objects Wrapper around PropertyList
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public abstract class AbstractWidgetState extends Observable {
    // button-related state
    protected PropertyList _properties;
    // protected Cls _cls;
    // protected KnowledgeBase _kb;

    public AbstractWidgetState(PropertyList properties) {
        _properties = properties;
        restore();
    }

    protected void broadcast() {
        setChanged();
        notifyObservers();
    }

    public abstract void dispose();

    protected boolean isCustomized() {
        // pretty dumb right now-- empty implies never been customized.
        Collection names = _properties.getNames();
        if ((null == names) || (0 == names.size())) {
            return false;
        }
        return true;
    }

    protected boolean readBoolean(String name, boolean defaultValue) {
        Boolean objectValue = _properties.getBoolean(name);
        if (null != objectValue) {
            return objectValue.booleanValue();
        }
        return defaultValue;
    }

    protected int readInt(String name, int defaultValue) {
        Integer objectValue = _properties.getInteger(name);
        if (null != objectValue) {
            return objectValue.intValue();
        }
        return defaultValue;
    }

    protected String readString(String name, String defaultValue) {
        String objectValue = _properties.getString(name);
        if (null != objectValue) {
            return objectValue;
        }
        return defaultValue;
    }

    public abstract void restore();

    public abstract void save();
}
