/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.abstracttable;

import java.util.*;

import javax.swing.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protegex.util.*;

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class BooleanEditor extends ComboBoxEditor {
    private static final long serialVersionUID = -1174955571929718873L;
    private ArrayList _choices;
    protected EditableComboBoxModel _model;
    protected Boolean _value;
    private static String TRUE = "true";
    private static String FALSE = "false";

    public BooleanEditor() {
        this(null, null);
    }

    public BooleanEditor(Instance instance, Slot slot) {
        super(instance, slot);
    }

    private void configureComboBoxModel() {
        if (null == _choices) {
            _choices = new ArrayList();
            _choices.add(NULL_STRING);
            _choices.add(TRUE);
            _choices.add(FALSE);
        }
        _model.setContents(_choices);
    }

    public void configureUIFromKB() {
        if (_readingFromModel) {
            return;
        }
        _readingFromModel = true;
        _value = (Boolean) getValue();
        configureComboBoxModel();
        selectValueInComboBox();
        _readingFromModel = false;
    }

    protected ComboBoxModel getComboBoxModel() {
        _model = new EditableComboBoxModel();
        return _model;
    }

    public void selectValueInComboBox() {
        Object value = getValue();
        if (null == value) {
            _model.setSelectedItem(NULL_STRING);
        } else {
            if ((Boolean.TRUE).equals(value)) {
                _model.setSelectedItem(TRUE);
            } else {
                _model.setSelectedItem(FALSE);
            }
        }
    }

    public void storeValueInKB() {
        if ((_instance == null) || (_slot == null)) {
            return;
        }
        Object value = _model.getSelectedItem();
        if (NULL_STRING == value) {
            _instance.setOwnSlotValue(_slot, null);
        } else {
            if (TRUE == value) {
                _instance.setOwnSlotValue(_slot, Boolean.TRUE);
            } else {
                _instance.setOwnSlotValue(_slot, Boolean.FALSE);
            }
        }
    }
}
