/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.abstracttable;

import javax.swing.*;

import edu.stanford.smi.protege.model.*;

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public abstract class ComboBoxEditor extends JComboBox implements TableEditorInterface {
    private static final long serialVersionUID = 5512576126000550787L;
    protected Instance _instance;
    protected Slot _slot;
    protected boolean _readingFromModel;
    protected boolean _processingSelection;

    public ComboBoxEditor() {
        this(null, null);
    }

    public ComboBoxEditor(Instance instance, Slot slot) {
        super();
        _processingSelection = false;
        _readingFromModel = false;
        _instance = instance;
        _slot = slot;
        setModel(getComboBoxModel());
        configureUIFromKB();
    }

    public abstract void configureUIFromKB();

    protected abstract ComboBoxModel getComboBoxModel();

    public Object getValue() {
        if ((null == _instance) || (null == _slot)) {
            return null;
        }
        return _instance.getOwnSlotValue(_slot);
    }

    protected void handleSelectionChange() {
        if (_processingSelection) {
            return;
        }
        _processingSelection = true;
        if (!_readingFromModel) {
            storeValueInKB();
        }
        _processingSelection = false;
    }

    /**
     * @deprecated
     */
    public boolean isManagingFocus() {
        return false;
    }

    public boolean isValueAcceptable() {
        return true;
    }

    public boolean needsToStoreChanges() {
        return false;
    }

    protected void selectedItemChanged() {
        handleSelectionChange();
    }

    public void setInstance(Instance instance) {
        _instance = instance;
        configureUIFromKB();
    }

    public void setSlot(Slot slot) {
        _slot = slot;
        configureUIFromKB();
    }

    public abstract void storeValueInKB();
}
