/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.abstracttable;

import java.util.*;

import javax.swing.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.ui.*;

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class InstanceEditor_UseDialog extends FrameEditor_UseDialog {
    private static final long serialVersionUID = -8143010746190054939L;
    private String _dialogTitle;

    public InstanceEditor_UseDialog(String dialogTitle, JComponent onScreenComponent, Project project) {
        this(dialogTitle, onScreenComponent, project, null, null);
    }

    public InstanceEditor_UseDialog(
        String dialogTitle,
        JComponent onScreenComponent,
        Project project,
        Instance instance,
        Slot slot) {
        super(onScreenComponent, project, instance, slot);
        _dialogTitle = dialogTitle;
    }

    protected void getValueFromDialog() {
        _processingUserSelection = true;
        if ((null != _instance) && (null != _slot)) {
            Cls cls = _instance.getDirectType();
            Collection allowedClasses = cls.getTemplateSlotAllowedClses(_slot);
            Instance selection = DisplayUtilities.pickInstance(_onScreenComponent, allowedClasses, _dialogTitle);
            Frame newValue = (Frame) selection;
            if (null != newValue) {
                _value = newValue;
                storeValueInKB();
                configureUIFromKB();
            }
        }
        _processingUserSelection = false;
        _onScreenComponent.repaint();
        return;
    }
}
