/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.abstracttable;

import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

/**
 *  Unused
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class TableEditorListenerManager {
    private ArrayList _listeners;
    private ChangeEvent _event;

    public TableEditorListenerManager(JTable underlyingTable) {
        _listeners = new ArrayList();
        _event = new ChangeEvent(underlyingTable);
    }

    public void addCellEditorListener(CellEditorListener l) {
        ArrayList newList = new ArrayList(_listeners);
        newList.add(l);
        _listeners = newList;
    }

    public void fireEditingCanceled() {
        Iterator i = _listeners.iterator();
        while (i.hasNext()) {
            CellEditorListener nextListener = (CellEditorListener) i.next();
            nextListener.editingCanceled(_event);
        }
    }

    public void fireEditingStopped() {
        Iterator i = _listeners.iterator();
        while (i.hasNext()) {
            CellEditorListener nextListener = (CellEditorListener) i.next();
            nextListener.editingStopped(_event);
        }
    }

    public void removeCellEditorListener(CellEditorListener l) {
        ArrayList newList = new ArrayList(_listeners);
        newList.remove(l);
        _listeners = newList;
    }
}
