/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.contains;

import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import edu.stanford.smi.protege.model.*;

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public abstract class ContainsWidget_AbstractAction extends AbstractAction {
    private static final long serialVersionUID = -718905515021290019L;
    protected ContainsWidget _widget;
    protected Project _project;
    protected KnowledgeBase _kb;
    protected ContainsWidgetState _state;

    public ContainsWidget_AbstractAction(ContainsWidget widget, String tooltipString, Icon icon) {
        super(tooltipString, icon);
        _project = widget.getProject();
        _state = widget.getState();
        _kb = widget.getKnowledgeBase();
        _widget = widget;
        updateActivation();
    }

    public abstract void actionPerformed(ActionEvent e);

    protected Collection getAllowedClasses() {
        Instance instance = _widget.getInstance();
        Slot slot = _widget.getSlot();
        Cls cls = instance.getDirectType();
        return new ArrayList(cls.getTemplateSlotAllowedClses(slot));
    }

    protected void updateActivation() {
        if (_widget.isSelectionEmpty()) {
            setEnabled(false);
        } else {
            setEnabled(true);
        }
    }
}
