/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.contains;

import java.awt.event.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.resource.*;
import edu.stanford.smi.protegex.util.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class ContainsWidget_PrototypeAction extends ContainsWidget_AbstractAction {
    private static final long serialVersionUID = 7517380533925014719L;
    private int _depth = 1;

    public ContainsWidget_PrototypeAction(ContainsWidget widget, int depth) {
        super(widget, (widget.getState()).getPrototypeButtonTooltip(), Icons.getCopyIcon());
        _depth = depth;
    }

    public void actionPerformed(ActionEvent e) {
        Instance selectedInstance = _widget.getSelectedInstance();
        if (null == selectedInstance) {
            return;
        }
        Instance newInstance = createUsingPrototype(selectedInstance, _depth);
        if (_state.isCreateFormForNewInstances()) {
            _project.show(newInstance);
        }
        return;
    }

    private Instance createUsingPrototype(Instance prototypeInstance, int depth) {
        return RecursiveCopy.recursivelyCopyInstance(prototypeInstance, depth);
    }

    protected void updateActivation() {
        super.updateActivation();
        if (!_widget.allowsMultipleValues()) {
            setEnabled(false);
        }
    }
}
