/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.contains;

import javax.swing.*;

import edu.stanford.smi.protege.event.*;
import edu.stanford.smi.protege.model.*;

/**
 *  Not available from UI
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public abstract class FormDescription extends JLabel {

    private static final long serialVersionUID = 2951715720232312763L;
    protected Instance _instance;
    private SentenceUpdater _sentenceUpdater;
    private String _description;

    private class SentenceUpdater implements FrameListener {
        public void browserTextChanged(FrameEvent event) {
        }

        public void deleted(FrameEvent event) {
        }

        public void editabilityChanged(FrameEvent event) {
        }

        public void nameChanged(FrameEvent event) {
        }

        public void ownFacetAdded(FrameEvent event) {
        }

        public void ownFacetRemoved(FrameEvent event) {
        }

        public void ownFacetValueChanged(FrameEvent event) {
        }

        public void ownSlotAdded(FrameEvent event) {
        }

        public void ownSlotRemoved(FrameEvent event) {
        }

        public void visibilityChanged(FrameEvent event) {
        }

        public void ownSlotValueChanged(FrameEvent event) {
            updateLabel();
        }
    }

    public FormDescription(Instance instance) {
        _sentenceUpdater = new SentenceUpdater();
        setHorizontalAlignment(SwingConstants.CENTER);
        setInstance(instance);
    }

    private void addListener() {
        if (null != _instance) {
            _instance.addFrameListener(_sentenceUpdater);
        }
    }

    protected abstract String getDescriptiveSentence();

    private void removeListener() {
        if (null != _instance) {
            _instance.removeFrameListener(_sentenceUpdater);
        }
    }

    public void setInstance(Instance instance) {
        removeListener();
        _instance = instance;
        updateLabel();
        addListener();
    }

    private void updateLabel() {
        _description = getDescriptiveSentence();
        setText(_description);
    }
}
