/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.contains;

import javax.swing.*;

/**
 *  Description of the Class
 *
 * @author William Grosso <grosso@smi.stanford.edu>
 */
public class HorizontalWidgetWrapperHolder extends WidgetWrapperHolder {

    public HorizontalWidgetWrapperHolder() {
        super(
            DEFAULT_SEPARATION_DISTANCE,
            DEFAULT_USE_SEPARATORS,
            DEFAULT_VERTICAL_SCROLL_INCREMENT,
            DEFAULT_HORIZONTAL_SCROLL_INCREMENT);
    }

    public HorizontalWidgetWrapperHolder(int spacerSize, boolean useSpacers) {
        super(spacerSize, useSpacers, DEFAULT_VERTICAL_SCROLL_INCREMENT, DEFAULT_HORIZONTAL_SCROLL_INCREMENT);
    }

    public HorizontalWidgetWrapperHolder(
        int spacerSize,
        boolean useSpacers,
        int verticalScrollIncrement,
        int horizontalScrollIncrement) {
        super(spacerSize, useSpacers, verticalScrollIncrement, horizontalScrollIncrement);
    }

    protected WidgetBox getWidgetBox(int spacerSize, boolean useSpacers) {
        return new WidgetBox(new HorizontalWidgetBoxLayoutManager(), spacerSize, useSpacers);
    }

    protected void scrollToCorrectLocation() {
        JScrollBar verticalScrollBarForScrollPane = _scrollPaneAroundWidgetBox.getVerticalScrollBar();
        if ((_selection != null) && (_selection.getParent() != null)) {
            _selectionRectangle = _selection.getBounds();
            _selectionRectangle.y += (_selection.getParent()).getY();
            _viewport.scrollRectToVisible(_selectionRectangle);
            _viewport.repaint();
        } else {
            verticalScrollBarForScrollPane.setValue(0);
        }
        return;
    }
}
