/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.contains;

import java.awt.event.*;

import javax.swing.*;

import edu.stanford.smi.protegex.util.*;

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class SummaryConfigurationPanel extends AbstractWidgetConfigurationPanel {
    private static final long serialVersionUID = 8964526206725058142L;
    private ContainsWidgetState _containsWidgetState;
    private JCheckBox _displaySummaryPanel;
    private JCheckBox _displayTotalNumberOfSubordinateInstances;
    private JCheckBox _displayBrowserKeysOfSubordinateInstances;

    private class DisplaySummaryInformationCheckboxListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            _containsWidgetState.setDisplaySummaryPanel(_displaySummaryPanel.isSelected());
        }
    }

    private class DisplayTotalNumberOfSubordinateInstancesCheckboxListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            _containsWidgetState.setDisplayTotalNumberOfSubordinateInstances(_displayTotalNumberOfSubordinateInstances.isSelected());
        }
    }

    private class DisplayBrowserKeysOfSubordinateInstancesCheckboxListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            _containsWidgetState.setDisplayBrowserKeysOfSubordinateInstances(_displayBrowserKeysOfSubordinateInstances.isSelected());
        }
    }

    public SummaryConfigurationPanel(ContainsWidgetState state) {
        super(state);
    }

    private void buildDisplayBrowserKeysOfSubordinateInstances(int yPosition) {
        _displayBrowserKeysOfSubordinateInstances = createCheckBox("Display Browser Keys of Subordinate Instances", yPosition);
        _displayBrowserKeysOfSubordinateInstances.setSelected(
            _containsWidgetState.isDisplayBrowserKeysOfSubordinateInstances());
        _displayBrowserKeysOfSubordinateInstances.addActionListener(
            new DisplayBrowserKeysOfSubordinateInstancesCheckboxListener());
    }

    private void buildDisplaySummaryPanel(int yPosition) {
        _displaySummaryPanel = createCheckBox("Display Summary Panel", yPosition);
        _displaySummaryPanel.setSelected(_containsWidgetState.isDisplaySummaryPanel());
        _displaySummaryPanel.addActionListener(new DisplaySummaryInformationCheckboxListener());
    }

    private void buildDisplayTotalNumberOfSubordinateInstances(int yPosition) {
        _displayTotalNumberOfSubordinateInstances = createCheckBox("Display Total Number of Subordinate Instances", yPosition);
        _displayTotalNumberOfSubordinateInstances.setSelected(
            _containsWidgetState.isDisplayTotalNumberOfSubordinateInstances());
        _displayTotalNumberOfSubordinateInstances.addActionListener(
            new DisplayTotalNumberOfSubordinateInstancesCheckboxListener());
    }

    protected void buildGUI() {
        _containsWidgetState = (ContainsWidgetState) _state;
        buildDisplaySummaryPanel(1);
        buildDisplayTotalNumberOfSubordinateInstances(2);
        buildDisplayBrowserKeysOfSubordinateInstances(3);
        addVerticalSpace(4);
    }
}
