/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.graph;

import edu.stanford.smi.protege.model.Frame;

/**
 * Encapsulates data object to be rendered in various combo boxes and tables 
 * in the widget configuration dialog.  The main reason for this is that OWL 
 * class names are long and we only want to display the short name in the 
 * user interface, but we still need access to the longer name when we store 
 * configuration properties in the PPRJ files.
 * 
 * @author Jennifer Vendetti <vendetti@stanford.edu>
 *
 */
public class FrameData {
	private Frame frame;
	private String displayName;
	private String fullName;
	
	public FrameData (Frame frame) {
		this.frame = frame;
		displayName = frame.getBrowserText();
		fullName = frame.getName();
	}

	public String getDisplayName() {
		return displayName;
	}

	public String getFullName() {
		return fullName;
	}
	
	public Frame getFrame() {
		return frame;
	}
	
	public String toString() {
		return displayName;
	}

	@Override
	public boolean equals(Object obj) {
		boolean retval = false;
		if (obj instanceof FrameData) {
			FrameData data = (FrameData) obj;
			if (this.fullName.equals(data.getFullName())) {
				retval = true;
			}
		}
		return retval;
	}
}