/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.graph;

import java.util.ArrayList;

import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Slot;
import edu.stanford.smi.protege.util.PropertyList;
import edu.stanford.smi.protege.widget.WidgetConfigurationPanel;

public class GraphConfigurationPanel extends WidgetConfigurationPanel {
    private static final long serialVersionUID = 2522822069038437603L;
    private NodeConfigurationPanel nodePanel;
    private RelationConfigurationPanel relationPanel;
    private ViewConfigurationPanel viewPanel;
    private Slot slot;
    private Cls cls;
    private PropertyList propertyList;

    public GraphConfigurationPanel(GraphWidget widget) {
        super(widget);

        this.slot = widget.getSlot();
        this.cls = widget.getCls();
        this.propertyList = widget.getPropertyList();

        initialize();
    }

    private void initialize() {
        ArrayList allowedClses = new ArrayList(cls.getTemplateSlotAllowedClses(slot));
        nodePanel = new NodeConfigurationPanel(allowedClses, propertyList);
        addTab("Nodes", nodePanel);

        relationPanel = new RelationConfigurationPanel(cls, propertyList);
        addTab("Reified Relations", relationPanel);

        viewPanel = new ViewConfigurationPanel(cls, propertyList);
        addTab("View Properties", viewPanel);
    }

    public void saveContents() {
        super.saveContents();
        nodePanel.saveContents();
        viewPanel.saveContents();
        if (relationPanel.hasReifiedRelations()) {
            relationPanel.saveContents();
        }
    }
}