/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.graph;

import edu.stanford.smi.protege.util.PropertyList;

public abstract class GraphProperties {
    private String clsName;
    private String prefix;
    private PropertyList propertyList;

    public GraphProperties(String clsName, PropertyList propertyList) {
        this.clsName = clsName;
        this.prefix = clsName + "_";
        this.propertyList = propertyList;
    }

    public abstract void save();

    public String getStringProperty(String property, String defaultValue) {
        String s = propertyList.getString(property);
        if (s == null) {
            s = defaultValue;
        }
        return s;
    }

    public boolean getBooleanProperty(String property, Boolean defaultValue) {
        Boolean b = propertyList.getBoolean(property);
        if (b == null) {
            b = defaultValue;
        }
        return b.booleanValue();
    }

    public int getIntProperty(String property, Integer defaultValue) {
        Integer i = propertyList.getInteger(property);
        if (i == null) {
            i = defaultValue;
        }
        return i.intValue();
    }

    public String getClsName() {
        return clsName;
    }

    public String getPrefix() {
        return prefix;
    }

    public PropertyList getPropertyList() {
        return propertyList;
    }
}