/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.imagemap;

import java.awt.*;
import java.util.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class ImageMapPanelEventSupport {
    protected ArrayList _listeners;

    public ImageMapPanelEventSupport() {
        _listeners = new ArrayList();
    }

    public void addImageMapPanelListener(ImageMapPanelListener newListener) {
        if (!_listeners.contains(newListener)) {
            ArrayList temp = new ArrayList(_listeners);
            temp.add(newListener);
            _listeners = temp;
        }
        return;
    }

    public void callLogicalClick(Point where) {
        Iterator i = _listeners.iterator();
        while (i.hasNext()) {
            ((ImageMapPanelListener) i.next()).logicalClick(where);
        }
        return;
    }

    public void callLogicalDrag(Point where) {
        Iterator i = _listeners.iterator();
        while (i.hasNext()) {
            ((ImageMapPanelListener) i.next()).logicalDrag(where);
        }
        return;
    }

    public void callLogicalDragFinished(Point where) {
        Iterator i = _listeners.iterator();
        while (i.hasNext()) {
            ((ImageMapPanelListener) i.next()).logicalDragFinished(where);
        }
        return;
    }

    public void callLogicalDragStarted(Point where) {
        Iterator i = _listeners.iterator();
        while (i.hasNext()) {
            ((ImageMapPanelListener) i.next()).logicalDragStarted(where);
        }
        return;
    }

    public void callLogicalPress(Point where) {
        Iterator i = _listeners.iterator();
        while (i.hasNext()) {
            ((ImageMapPanelListener) i.next()).logicalPress(where);
        }
        return;
    }

    public void callLogicalRelease(Point where) {
        Iterator i = _listeners.iterator();
        while (i.hasNext()) {
            ((ImageMapPanelListener) i.next()).logicalRelease(where);
        }
        return;
    }

    public void removeImageMapPanelListener(ImageMapPanelListener oldListener) {
        if (_listeners.contains(oldListener)) {
            ArrayList temp = new ArrayList(_listeners);
            temp.remove(oldListener);
            _listeners = temp;
        }
        return;
    }
}
