/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.imagemap;

import javax.swing.*;
import java.awt.*;

/**
 *  Description of the Class Line whose width is equal to the value of the
 *  annotated slider position
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class LineView extends JPanel implements SecondaryView {
    private static final long serialVersionUID = 4138435177070044265L;
    private Color _color;
    private Insets _insets;
    private Dimension _size;
    private BasicStroke _stroke;
    public static Color DEFAULT_COLOR = Color.black;

    public LineView() {
        this(DEFAULT_COLOR);
    }

    public LineView(Color color) {
        _color = color;
        _insets = new Insets(0, 0, 0, 0);
        _size = new Dimension(0, 0);
        setPreferredSize(new Dimension(40, 40));
        setOpaque(true);
    }

    public void paint(Graphics g) {
        super.paint(g);
        if (g instanceof Graphics2D) {
            twoDPaint((Graphics2D) g);
        }
    }

    public void setValue(int value) {
        _stroke = new BasicStroke((float) value);
        repaint();
    }

    private void twoDPaint(Graphics2D g) {
        if (null == _stroke) {
            return;
        }
        getInsets(_insets);
        getSize(_size);
        int startingY = _insets.top;
        int endingY = _size.height - (_insets.top + _insets.bottom);
        int xValue = _insets.left + ((int) (.5 * _size.width));
        g.setColor(_color);
        g.setStroke(_stroke);
        g.drawLine(xValue, startingY, xValue, endingY);
    }
}
