/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.instancetable;

import java.awt.event.*;
import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.resource.*;
import edu.stanford.smi.protegex.util.*;

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class Action_CreateUsingPrototype extends Action_AbstractTableListener {

    private static final long serialVersionUID = -4297575243479509311L;

    public Action_CreateUsingPrototype(InstanceTableWidget widget, InstanceTable table) {
        super((widget.getState()).getPrototypeButtonTooltip(), Icons.getCopyIcon(), widget, table);
    }

    public void actionPerformed(ActionEvent e) {
        if (isSlotEditable()) {
            Collection instances = _table.getSelectedInstances();
            Iterator i = instances.iterator();
            int depth = (_widget.getState()).getPrototypeDepth();
            ArrayList newInstances = new ArrayList();
            Instance lastInstance = null;
            while (i.hasNext()) {
                lastInstance = createUsingPrototype((Instance) i.next(), depth);
                newInstances.add(lastInstance);
            }
            _widget.addValues(newInstances);
            if (null != lastInstance) {
                _widget.selectInstanceIfAppropriate(lastInstance);
            }
            if ((_widget.getState()).isCreateFormForNewInstances()) {
                showInstances(newInstances);
            }
        }
    }

    private Instance createUsingPrototype(Instance prototypeInstance, int depth) {
        return RecursiveCopy.recursivelyCopyInstance(prototypeInstance, depth);
    }

    private void showInstances(Collection instances) {
        Project project = _widget.getProject();
        Iterator i = instances.iterator();
        while (i.hasNext()) {
            project.show((Instance) i.next());
        }
        return;
    }
}
