/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.instancetable;

import java.awt.*;

import javax.swing.*;
import javax.swing.table.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protegex.util.*;

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class ConfigTableRenderer extends JTextField implements TableCellRenderer {
    private static final long serialVersionUID = -4212694010918756148L;
    private ColorPanel _colorPanel;

    public ConfigTableRenderer() {
        setOpaque(true);
        _colorPanel = new ColorPanel(Color.white);
    }

    private Component configureForSlot(Slot slot) {
        setBackground(Color.white);
        setText(slot.getBrowserText());
        return this;
    }

    private Component configureForString(String text, int column) {
        if (0 == column) {
            setBackground(Color.lightGray);
        } else {
            setBackground(Color.white);
        }
        setText(text);
        return this;
    }

    public Component getTableCellRendererComponent(
        JTable table,
        Object value,
        boolean isSelected,
        boolean hasFocus,
        int row,
        int column) {
        if (value instanceof Color) {
            _colorPanel.setColor((Color) value);
            return _colorPanel;
        }
        if (value instanceof Slot) {
            return configureForSlot((Slot) value);
        }
        if (value instanceof String) {
            return configureForString((String) value, column);
        }
        // Should never get here. But, if we do, it will be obvious
        setBackground(Color.red);
        setText("Daughter of Ugly Widget");
        return this;
    }
}
