/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.scatterbox;

import java.awt.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.resource.*;
import edu.stanford.smi.protege.ui.*;
import edu.stanford.smi.protege.util.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class AddButton extends ScatterboxAction {
    private static final long serialVersionUID = 4063798177558405497L;
    private String _dialogTitle;

    public AddButton(
        ScatterboxWidget widget,
        ScatterboxTable table,
        ScatterboxTableModel model,
        KBQueryUtils queryUtilsObject) {
        super(widget, (widget.getState()).getAddInstanceButtonTooltip(), Icons.getAddIcon(), table, model, queryUtilsObject);
        _dialogTitle = (widget.getState()).getAddInstanceDialogTitle();
    }

    protected void performTask(int row, int column) {
        Cls rangeType = _queryUtilsObject.getRangeType(_widget.getCls());
        Instance value =
            DisplayUtilities.pickInstance(
                (Component) _widget,
                CollectionUtilities.createList(rangeType),
                "Choose a value to enter in the table");
        if (null != value) {
            Instance newEntry = _model.createEntry(row, column, value);
            _widget.addEntry(newEntry);
        }
        return;
    }

    protected void setActivation() {
        if (_widget.isThereASelection()) {
            if (null != _widget.getSelectedEntry()) {
                setEnabled(false);
            } else {
                setEnabled(true);
            }
            return;
        }
        setEnabled(false);
        return;
    }
}
