/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.scatterbox;

import edu.stanford.smi.protege.model.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class DomainRangeObjectPairEntryScatterboxConfigurationChecks implements DomainRangeObjectPairConstants {

    private static boolean checkEntryClsTaxonomicStructure(KBQueryUtils queryObject, Cls entryCls) {
        KnowledgeBase kb = entryCls.getKnowledgeBase();
        Cls entryMetaClass = kb.getCls(ENTRY_METACLASS_NAME);
        if (!entryCls.hasType(entryMetaClass)) {
            return false;
        }
        if (!StandardScatterboxConfigurationChecks.goodDomainSpecification(queryObject, entryCls)) {
            return false;
        }
        if (!StandardScatterboxConfigurationChecks.goodRangeSpecification(queryObject, entryCls)) {
            return false;
        }
        return true;
    }

    public static boolean isSuitable(Cls cls, Slot slot, Facet facet, KBQueryUtils queryObject) {
        if ((null == cls) || (null == slot)) {
            return false;
        }
        if (!StandardScatterboxConfigurationChecks.checkClsTaxonomicStructure(cls)) {
            return false;
        }
        Cls entryCls = queryObject.getEntryCls(cls);
        if ((null == entryCls) || !checkEntryClsTaxonomicStructure(queryObject, entryCls)) {
            return false;
        }
        return true;
    }
}
