/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.scatterbox;

import edu.stanford.smi.protege.model.*;
import java.util.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class Instance_Filling extends BasicOrder implements Constants {

    public Instance_Filling(ScatterboxWidget widget, Slot slot, KBQueryUtils queryUtilsObject) {
        super(widget, slot, queryUtilsObject);
    }

    private ArrayList getInstances(String policy, Collection hierarchicalRoots) {
        if (null == hierarchicalRoots) {
            return null;
        }
        if (policy.equals(DIRECT_INSTANCES_ONLY)) {
            return returnDirectInstances(hierarchicalRoots);
        }
        return returnAllInstances(hierarchicalRoots);
    }

    protected Collection getValues() {
        Cls domainCls = _queryUtilsObject.getDomainType(_widget.getCls());
        ScatterboxWidgetState state = _widget.getState();
        Collection hierarchicalRoots = domainCls.getTemplateSlotAllowedClses(_slot);
        String policy = state.getPolicyWhenInstancesAreDomainIndices();
        ArrayList instances = getInstances(policy, hierarchicalRoots);
        return instances;
    }

    private ArrayList returnAllInstances(Collection hierarchicalRoots) {
        ArrayList instanceList = new ArrayList();
        Iterator i = hierarchicalRoots.iterator();
        while (i.hasNext()) {
            Cls nextCls = (Cls) i.next();
            ArrayList nextList = new ArrayList(nextCls.getInstances());
            instanceList.add(nextList);
        }
        return concatenateWithoutDuplicates(instanceList);
    }

    private ArrayList returnDirectInstances(Collection hierarchicalRoots) {
        ArrayList instanceList = new ArrayList();
        Iterator i = hierarchicalRoots.iterator();
        while (i.hasNext()) {
            Cls nextCls = (Cls) i.next();
            ArrayList nextList = new ArrayList(nextCls.getDirectInstances());
            instanceList.add(nextList);
        }
        return concatenateWithoutDuplicates(instanceList);
    }
}
