/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.scatterbox;

import java.awt.*;

import javax.swing.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protegex.widget.abstracttable.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class ScatterboxTableCellEditor extends AbstractTableWidgetCellEditor {
    private AbstractTableWidgetValue _abstractTableWidgetValue;
    private ScatterboxWidget _widget;
    private ScatterboxWidgetState _widgetState;
    private ScatterboxTableModel _model;
    private KBQueryUtils _queryUtilsObject;

    public ScatterboxTableCellEditor(
        ScatterboxWidget widget,
        ScatterboxWidgetState widgetState,
        JTable underlyingTable,
        Project project,
        KBQueryUtils queryUtilsObject) {
        super(underlyingTable, widgetState, project);
        _queryUtilsObject = queryUtilsObject;
        _model = (ScatterboxTableModel) underlyingTable.getModel();
        _widget = widget;
        _widgetState = widgetState;
        _abstractTableWidgetValue = new AbstractTableWidgetValue();
        _abstractTableWidgetValue.color = Color.black;
    }

    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
        if (null == value) {
            value = _model.createEntry(row, column);
            _widget.addEntry((Instance) value);
        }
        Instance entryInstance = (Instance) value;
        Instance rangeInstance = _queryUtilsObject.getRangeObject(entryInstance);
        _abstractTableWidgetValue.instance = rangeInstance;
        _abstractTableWidgetValue.slot = _queryUtilsObject.getRangeValueSlot(entryInstance);
        _abstractTableWidgetValue.values = rangeInstance.getOwnSlotValues(_abstractTableWidgetValue.slot);
        return super.getTableCellEditorComponent(table, _abstractTableWidgetValue, isSelected, row, column);
    }
}
