/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.widget.scatterbox;

import java.awt.*;
import java.awt.dnd.*;
import java.awt.event.*;
import java.util.*;

import edu.stanford.smi.protege.model.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class ScatterboxTableDragGestureListener extends ScatterboxDNDObject
         implements DragGestureListener {

    public ScatterboxTableDragGestureListener(
        ScatterboxWidget widget,
        ScatterboxTable underlyingTable,
        KBQueryUtils queryUtilsObject) {
        super(widget, underlyingTable, queryUtilsObject);
    }

    public void dragGestureRecognized(DragGestureEvent dragGestureEvent) {
        if (!isThisAnAcceptableEvent(dragGestureEvent)) {
            return;
        }
        Instance entry = getEntryForDragGestureEvent(dragGestureEvent);
        if (null != entry) {
            TransferableCell transferableCell = new TransferableCell(entry, _underlyingTable);
            _underlyingTable.stopEditing();
            dragGestureEvent.startDrag(
                DragSource.DefaultMoveDrop,
                transferableCell,
                new ScatterboxTableDragSourceListener(_widget, _underlyingTable, _queryUtilsObject));
        }
        return;
    }

    private Instance getEntryForDragGestureEvent(DragGestureEvent dragGestureEvent) {
        Point dragStartPoint = getTableLocationForEvent(dragGestureEvent);
        if (dragStartPoint == null) {
            return null;
        }
        Instance entry = (Instance) _underlyingTable.getValueAt(dragStartPoint.x, dragStartPoint.y);
        return entry;
    }

    private boolean isThisAnAcceptableEvent(DragGestureEvent dragGestureEvent) {
        Iterator i = dragGestureEvent.iterator();
        if (!i.hasNext()) {
            return false;
        }
        MouseEvent firstEvent = (MouseEvent) i.next();
        if (!i.hasNext()) {
            return false;
        }
        MouseEvent secondEvent = (MouseEvent) i.next();
        if ((firstEvent.getID() == MouseEvent.MOUSE_PRESSED) && (secondEvent.getID() == MouseEvent.MOUSE_DRAGGED)) {
            return true;
        }
        return false;
    }
}
